=== modified file 'NEWS'
--- NEWS	2005-12-12 23:14:43 +0000
+++ NEWS	2005-12-14 07:53:26 +0000
@@ -172,7 +172,12 @@
       and once with all locales forced to C, to expose bugs. This is not 
       trivially done within python, so for now its only triggered by running
       Make check. Integrators and packagers who wish to check for full 
-      platform support should run 'Make check' to test the source.
+      platform support should run 'make check' to test the source.
+      (Robert Collins)
+
+    * Tests can now run TestSkipped if they can't execute for any reason.
+      (Martin Pool) (NB: TestSkipped should only be raised for correctable
+      reasons - see the wiki spec ImprovingBzrTestSuite).
 
   INTERNALS:
 

=== modified file 'bzrlib/tests/__init__.py'
--- bzrlib/tests/__init__.py	2005-12-12 07:20:02 +0000
+++ bzrlib/tests/__init__.py	2005-12-13 05:46:40 +0000
@@ -117,6 +117,8 @@
         self._start_time = time.time()
 
     def addError(self, test, err):
+        if isinstance(err[1], TestSkipped):
+            return self.addSkipped(test, err)    
         unittest.TestResult.addError(self, test, err)
         if self.showAll:
             self.stream.writeln("ERROR %s" % self._elapsedTime())
@@ -140,14 +142,28 @@
         self.stream.flush()
         unittest.TestResult.addSuccess(self, test)
 
+    def addSkipped(self, test, skip_excinfo):
+        if self.showAll:
+            print >>self.stream, ' SKIP %s' % self._elapsedTime()
+            print >>self.stream, '     %s' % skip_excinfo[1]
+        elif self.dots:
+            self.stream.write('S')
+        self.stream.flush()
+        # seems best to treat this as success from point-of-view of unittest
+        # -- it actually does nothing so it barely matters :)
+        unittest.TestResult.addSuccess(self, test)
+
     def printErrorList(self, flavour, errors):
         for test, err in errors:
             self.stream.writeln(self.separator1)
             self.stream.writeln("%s: %s" % (flavour,self.getDescription(test)))
             if hasattr(test, '_get_log'):
-                self.stream.writeln()
-                self.stream.writeln('log from this test:')
+                print >>self.stream
+                print >>self.stream, \
+                        ('vvvv[log from %s]' % test).ljust(78,'-')
                 print >>self.stream, test._get_log()
+                print >>self.stream, \
+                        ('^^^^[log from %s]' % test).ljust(78,'-')
             self.stream.writeln(self.separator2)
             self.stream.writeln("%s" % err)
 

=== modified file 'bzrlib/tests/test_msgeditor.py'
--- bzrlib/tests/test_msgeditor.py	2005-12-12 23:33:03 +0000
+++ bzrlib/tests/test_msgeditor.py	2005-12-14 07:53:26 +0000
@@ -21,7 +21,7 @@
 
 from bzrlib.branch import Branch
 from bzrlib.msgeditor import make_commit_message_template
-from bzrlib.tests import TestCaseInTempDir
+from bzrlib.tests import TestCaseInTempDir, TestSkipped
 
 
 class MsgEditorTest(TestCaseInTempDir):
@@ -31,7 +31,11 @@
         b = Branch.initialize('.')
         working_tree = b.working_tree()
         filename = u'hell\u00d8'
-        self.build_tree_contents([(filename, 'contents of hello')])
+        try:
+            self.build_tree_contents([(filename, 'contents of hello')])
+        except UnicodeEncodeError:
+            raise TestSkipped("can't build unicode working tree in "
+                "filesystem encoding %s" % sys.getfilesystemencoding())
         working_tree.add(filename)
         return working_tree
     

=== modified file 'bzrlib/tests/test_selftest.py'
--- bzrlib/tests/test_selftest.py	2005-12-12 23:14:43 +0000
+++ bzrlib/tests/test_selftest.py	2005-12-14 07:53:26 +0000
@@ -18,8 +18,15 @@
 
 import os
 import sys
+import unittest
 
-from bzrlib.tests import TestCase, TestCaseInTempDir, _load_module_by_name
+from bzrlib.tests import (
+                          _load_module_by_name,
+                          TestCase,
+                          TestCaseInTempDir,
+                          TestSkipped,
+                          TextTestRunner,
+                          )
 
 
 class SelftestTests(TestCase):
@@ -49,3 +56,15 @@
         filename = u'hell\u00d8'
         self.build_tree_contents([(filename, 'contents of hello')])
         self.failUnlessExists(filename)
+
+
+class TestSkippedTest(TestCase):
+    """Try running a test which is skipped, make sure it's reported properly."""
+    def test_skipped_test(self):
+        # must be hidden in here so it's not run as a real test
+        def skipping_test():
+            raise TestSkipped('test intentionally skipped')
+        runner = TextTestRunner(stream=self._log_file)
+        test = unittest.FunctionTestCase(skipping_test)
+        result = runner.run(test)
+        self.assertTrue(result.wasSuccessful())

=== modified file 'bzrlib/weave.py'
--- bzrlib/weave.py	2005-12-08 06:18:00 +0000
+++ bzrlib/weave.py	2005-12-13 03:54:33 +0000
@@ -695,7 +695,11 @@
         lines_a = []
         lines_b = []
         ch_a = ch_b = False
-
+        # TODO: Return a structured form of the conflicts (e.g. 2-tuples for
+        # conflicted regions), rather than just inserting the markers.
+        # 
+        # TODO: Show some version information (e.g. author, date) on 
+        # conflicted regions.
         for state, line in plan:
             if state == 'unchanged' or state == 'killed-both':
                 # resync and flush queued conflicts changes if any
@@ -739,8 +743,9 @@
                                  'killed-both'), \
                        state
 
-                
+
     def join(self, other):
+        import sys, time
         """Integrate versions from other into this weave.
 
         The resulting weave contains all the history of both weaves; 
@@ -756,18 +761,37 @@
         # will be ok
         # work through in index order to make sure we get all dependencies
         for other_idx, name in enumerate(other._names):
-            if self._check_version_consistent(other, other_idx, name):
-                continue
+            self._check_version_consistent(other, other_idx, name)
+
+        merged = 0
+        processed = 0
+        time0 = time.time( )
         for other_idx, name in enumerate(other._names):
-            # TODO: If all the parents of the other version are already 
+            # TODO: If all the parents of the other version are already
             # present then we can avoid some work by just taking the delta
             # and adjusting the offsets.
             new_parents = self._imported_parents(other, other_idx)
+            sha1 = other._sha1s[other_idx]
+
+            processed += 1
+           
+            if name in self._names:
+                idx = self.lookup(name)
+                n1 = map(other.idx_to_name, other._parents[other_idx] )
+                n2 = map(self.idx_to_name, self._parents[other_idx] )
+                if sha1 ==  self._sha1s[idx] and n1 == n2:
+                        continue
+
+            merged += 1
             lines = other.get_lines(other_idx)
-            sha1 = other._sha1s[other_idx]
             self.add(name, new_parents, lines, sha1)
 
-
+        mutter("merged = %d, processed = %d, file_id=%s; deltat=%d"%(
+                merged,processed,self._weave_name, time.time( )-time0))
+
+
+
+ 
     def _imported_parents(self, other, other_idx):
         """Return list of parents in self corresponding to indexes in other."""
         new_parents = []

